@@ -1,8 +1,8 @@
 /*
 **  OSSP var - Variable Expansion
-**  Copyright (c) 2001-2002 Ralf S. Engelschall <rse@engelschall.com>
-**  Copyright (c) 2001-2002 The OSSP Project (http://www.ossp.org/)
-**  Copyright (c) 2001-2002 Cable & Wireless Deutschland (http://www.cw.com/de/)
+**  Copyright (c) 2001-2003 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 2001-2003 The OSSP Project (http://www.ossp.org/)
+**  Copyright (c) 2001-2003 Cable & Wireless Deutschland (http://www.cw.com/de/)
 **
 **  This file is part of OSSP var, a variable expansion
 **  library which can be found at http://www.ossp.org/pkg/lib/var/.
@@ -51,7 +51,7 @@
 
 /* unique library identifier */
 const char var_id[] = "OSSP var";
-        
+
 /* support for OSSP ex based exception throwing */
 #ifdef WITH_EX
 #include "ex.h"
@@ -113,13 +113,13 @@ static const var_syntax_t var_syntax_default = {
 */
 
 /* minimal output-independent vprintf(3) variant which supports %{c,s,d,%} only */
-static int 
+static int
 var_mvxprintf(
-    int (*output)(void *ctx, const char *buffer, size_t bufsize), void *ctx, 
+    int (*output)(void *ctx, const char *buffer, size_t bufsize), void *ctx,
     const char *format, va_list ap)
 {
     /* sufficient integer buffer: <available-bits> x log_10(2) + safety */
-    char ibuf[((sizeof(int)*8)/3)+10]; 
+    char ibuf[((sizeof(int)*8)/3)+10];
     char *cp;
     char c;
     int d;
@@ -190,9 +190,9 @@ typedef struct {
 } var_mvsnprintf_cb_t;
 
 /* output callback function for var_mvsnprintf() */
-static int 
+static int
 var_mvsnprintf_cb(
-    void *_ctx, 
+    void *_ctx,
     const char *buffer, size_t bufsize)
 {
     var_mvsnprintf_cb_t *ctx = (var_mvsnprintf_cb_t *)_ctx;
@@ -206,9 +206,9 @@ var_mvsnprintf_cb(
 }
 
 /* minimal vsnprintf(3) variant which supports %{c,s,d} only */
-static int 
+static int
 var_mvsnprintf(
-    char *buffer, size_t bufsize, 
+    char *buffer, size_t bufsize,
     const char *format, va_list ap)
 {
     int n;
@@ -218,7 +218,7 @@ var_mvsnprintf(
         return -1;
     if (buffer != NULL && bufsize == 0)
         return -1;
-    if (buffer == NULL) 
+    if (buffer == NULL)
         /* just determine output length */
         n = var_mvxprintf(NULL, NULL, format, ap);
     else {
@@ -274,7 +274,7 @@ typedef struct {
     size_t buffer_size;
 } tokenbuf_t;
 
-static void 
+static void
 tokenbuf_init(
     tokenbuf_t *buf)
 {
@@ -284,7 +284,7 @@ tokenbuf_init(
     return;
 }
 
-static int 
+static int
 tokenbuf_isundef(
     tokenbuf_t *buf)
 {
@@ -293,7 +293,7 @@ tokenbuf_isundef(
     return 0;
 }
 
-static int 
+static int
 tokenbuf_isempty(
     tokenbuf_t *buf)
 {
@@ -302,7 +302,7 @@ tokenbuf_isempty(
     return 0;
 }
 
-static void 
+static void
 tokenbuf_set(
     tokenbuf_t *buf, const char *begin, const char *end, size_t buffer_size)
 {
@@ -312,7 +312,7 @@ tokenbuf_set(
     return;
 }
 
-static void 
+static void
 tokenbuf_move(
     tokenbuf_t *src, tokenbuf_t *dst)
 {
@@ -323,13 +323,13 @@ tokenbuf_move(
     return;
 }
 
-static int 
+static int
 tokenbuf_assign(
     tokenbuf_t *buf, const char *data, size_t len)
 {
     char *p;
 
-    if ((p = malloc(len + 1)) == NULL)
+    if ((p = (char *)malloc(len + 1)) == NULL)
         return 0;
     memcpy(p, data, len);
     buf->begin = p;
@@ -339,7 +339,7 @@ tokenbuf_assign(
     return 1;
 }
 
-static int 
+static int
 tokenbuf_append(
     tokenbuf_t *output, const char *data, size_t len)
 {
@@ -350,7 +350,7 @@ tokenbuf_append(
     /* Is the tokenbuffer initialized at all? If not, allocate a
        standard-sized buffer to begin with. */
     if (output->begin == NULL) {
-        if ((output->begin = output->end = malloc(TOKENBUF_INITIAL_BUFSIZE)) == NULL)
+        if ((output->begin = output->end = (char *)malloc(TOKENBUF_INITIAL_BUFSIZE)) == NULL)
             return 0;
         output->buffer_size = TOKENBUF_INITIAL_BUFSIZE;
     }
@@ -365,7 +365,7 @@ tokenbuf_append(
         }
         /* ok, so copy the contents of output into an allocated buffer
            so that we can append that way. */
-        if ((tmp = malloc(output->end - output->begin + len + 1)) == NULL)
+        if ((tmp = (char *)malloc(output->end - output->begin + len + 1)) == NULL)
             return 0;
         memcpy(tmp, output->begin, output->end - output->begin);
         output->buffer_size = output->end - output->begin;
@@ -381,7 +381,7 @@ tokenbuf_append(
         do {
             new_size *= 2;
         } while ((new_size - (output->end - output->begin)) <= len);
-        if ((new_buffer = realloc((char *)output->begin, new_size)) == NULL)
+        if ((new_buffer = (char *)realloc((char *)output->begin, new_size)) == NULL)
             return 0;
         output->end = new_buffer + (output->end - output->begin);
         output->begin = new_buffer;
@@ -396,14 +396,14 @@ tokenbuf_append(
     return 1;
 }
 
-static int 
+static int
 tokenbuf_merge(
     tokenbuf_t *output, tokenbuf_t *input)
 {
     return tokenbuf_append(output, input->begin, input->end - input->begin);
 }
 
-static void 
+static void
 tokenbuf_free(
     tokenbuf_t *buf)
 {
@@ -420,36 +420,36 @@ tokenbuf_free(
 **
 */
 
-static void 
+static void
 expand_range(
-    char a, char b, char_class_t class)
+    char a, char b, char_class_t cclass)
 {
     do {
-        class[(int)a] = 1;
+        cclass[(int)a] = 1;
     }
     while (++a <= b);
     return;
 }
 
-static var_rc_t 
+static var_rc_t
 expand_character_class(
-    const char *desc, char_class_t class)
+    const char *desc, char_class_t cclass)
 {
     size_t i;
 
     /* clear the class array. */
     for (i = 0; i < 256; ++i)
-        class[i] = 0;
+        cclass[i] = 0;
 
     /* walk through class description and set appropriate entries in array */
     while (*desc != NUL) {
         if (desc[1] == '-' && desc[2] != NUL) {
             if (desc[0] > desc[2])
                 return VAR_ERR_INCORRECT_CLASS_SPEC;
-            expand_range(desc[0], desc[2], class);
+            expand_range(desc[0], desc[2], cclass);
             desc += 3;
         } else {
-            class[(int) *desc] = 1;
+            cclass[(int) *desc] = 1;
             desc++;
         }
     }
@@ -462,7 +462,7 @@ expand_character_class(
 **
 */
 
-static int 
+static int
 expand_isoct(
     char c)
 {
@@ -472,7 +472,7 @@ expand_isoct(
         return 0;
 }
 
-static var_rc_t 
+static var_rc_t
 expand_octal(
     const char **src, char **dst, const char *end)
 {
@@ -480,8 +480,8 @@ expand_octal(
 
     if (end - *src < 3)
         return VAR_ERR_INCOMPLETE_OCTAL;
-    if (   !expand_isoct(**src) 
-        || !expand_isoct((*src)[1]) 
+    if (   !expand_isoct(**src)
+        || !expand_isoct((*src)[1])
         || !expand_isoct((*src)[2]))
         return VAR_ERR_INVALID_OCTAL;
 
@@ -502,7 +502,7 @@ expand_octal(
     return VAR_OK;
 }
 
-static int 
+static int
 expand_ishex(
     char c)
 {
@@ -513,7 +513,7 @@ expand_ishex(
         return 0;
 }
 
-static var_rc_t 
+static var_rc_t
 expand_simple_hex(
     const char **src, char **dst, const char *end)
 {
@@ -521,15 +521,15 @@ expand_simple_hex(
 
     if (end - *src < 2)
         return VAR_ERR_INCOMPLETE_HEX;
-    if (   !expand_ishex(**src) 
+    if (   !expand_ishex(**src)
         || !expand_ishex((*src)[1]))
         return VAR_ERR_INVALID_HEX;
 
     if (**src >= '0' && **src <= '9')
         c = **src - '0';
-    else if (c >= 'a' && c <= 'f')
+    else if (**src >= 'a' && **src <= 'f')
         c = **src - 'a' + 10;
-    else if (c >= 'A' && c <= 'F')
+    else if (**src >= 'A' && **src <= 'F')
         c = **src - 'A' + 10;
 
     c = c << 4;
@@ -542,12 +542,12 @@ expand_simple_hex(
     else if (**src >= 'A' && **src <= 'F')
         c += **src - 'A' + 10;
 
-    **dst = (char) c;
+    **dst = (char)c;
     (*dst)++;
     return VAR_OK;
 }
 
-static var_rc_t 
+static var_rc_t
 expand_grouped_hex(
     const char **src, char **dst, const char *end)
 {
@@ -564,7 +564,7 @@ expand_grouped_hex(
     return VAR_OK;
 }
 
-static var_rc_t 
+static var_rc_t
 expand_hex(
     const char **src, char **dst, const char *end)
 {
@@ -577,7 +577,7 @@ expand_hex(
         return expand_simple_hex(src, dst, end);
 }
 
-/* 
+/*
 **
 **  ==== RECURSIVE-DESCEND VARIABLE EXPANSION PARSER ====
 **
@@ -589,7 +589,7 @@ static int parse_numexp (var_t *var, var_parse_t *ctx, const char *begin, const
 static int parse_name   (var_t *var, var_parse_t *ctx, const char *begin, const char *end);
 
 /* parse pattern text */
-static int 
+static int
 parse_pattern(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
@@ -608,7 +608,7 @@ parse_pattern(
 }
 
 /* parse substitution text */
-static int 
+static int
 parse_substext(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
@@ -627,7 +627,7 @@ parse_substext(
 }
 
 /* parse expression text */
-static int 
+static int
 parse_exptext(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
@@ -649,7 +649,7 @@ parse_exptext(
 }
 
 /* parse opertion argument text */
-static int 
+static int
 parse_opargtext(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
@@ -667,7 +667,7 @@ parse_opargtext(
     return (p - begin);
 }
 
-static int 
+static int
 parse_opargtext_or_variable(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -702,6 +702,7 @@ parse_opargtext_or_variable(
                 rc = VAR_ERR_OUT_OF_MEMORY;
                 goto error_return;
             }
+            tokenbuf_free(&tmp);
         }
     } while (rc > 0);
     tokenbuf_free(&tmp);
@@ -714,7 +715,7 @@ parse_opargtext_or_variable(
 }
 
 /* parse expression or variable */
-static int 
+static int
 parse_exptext_or_variable(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -747,11 +748,11 @@ parse_exptext_or_variable(
             goto error_return;
         if (rc > 0) {
             p += rc;
-            if (!tokenbuf_append
-                (result, tmp.begin, tmp.end - tmp.begin)) {
+            if (!tokenbuf_merge(result, &tmp)) {
                 rc = VAR_ERR_OUT_OF_MEMORY;
                 goto error_return;
             }
+            tokenbuf_free(&tmp);
         }
     } while (rc > 0);
 
@@ -765,7 +766,7 @@ parse_exptext_or_variable(
 }
 
 /* parse substitution text or variable */
-static int 
+static int
 parse_substext_or_variable(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -798,11 +799,11 @@ parse_substext_or_variable(
             goto error_return;
         if (rc > 0) {
             p += rc;
-            if (!tokenbuf_append
-                (result, tmp.begin, tmp.end - tmp.begin)) {
+            if (!tokenbuf_merge(result, &tmp)) {
                 rc = VAR_ERR_OUT_OF_MEMORY;
                 goto error_return;
             }
+            tokenbuf_free(&tmp);
         }
     } while (rc > 0);
 
@@ -816,13 +817,13 @@ parse_substext_or_variable(
 }
 
 /* parse class description */
-static int 
+static int
 parse_class_description(
     var_t *var, var_parse_t *ctx,
     tokenbuf_t *src, tokenbuf_t *dst)
 {
     unsigned char c, d;
-    const char *p; 
+    const char *p;
 
     p = src->begin;
     while (p != src->end) {
@@ -844,12 +845,12 @@ parse_class_description(
 }
 
 /* parse regex replace part */
-static int 
+static int
 parse_regex_replace(
     var_t *var, var_parse_t *ctx,
-    const char *data, 
+    const char *data,
     tokenbuf_t *orig,
-    regmatch_t *pmatch, 
+    regmatch_t *pmatch,
     tokenbuf_t *expanded)
 {
     const char *p;
@@ -901,17 +902,17 @@ parse_regex_replace(
 }
 
 /* operation: transpose */
-static int 
+static int
 op_transpose(
     var_t *var, var_parse_t *ctx,
-    tokenbuf_t *data, 
+    tokenbuf_t *data,
     tokenbuf_t *search,
     tokenbuf_t *replace)
 {
     tokenbuf_t srcclass, dstclass;
     const char *p;
     int rc;
-    size_t i;
+    int i;
 
     tokenbuf_init(&srcclass);
     tokenbuf_init(&dstclass);
@@ -936,7 +937,7 @@ op_transpose(
         tokenbuf_move(&tmp, data);
     }
     for (p = data->begin; p != data->end; ++p) {
-        for (i = 0; i <= (srcclass.end - srcclass.begin); ++i) {
+        for (i = 0; i <= (int)(srcclass.end - srcclass.begin); i++) {
             if (*p == srcclass.begin[i]) {
                 *((char *)p) = dstclass.begin[i];
                 break;
@@ -956,12 +957,12 @@ op_transpose(
 }
 
 /* operation: search & replace */
-static int 
+static int
 op_search_and_replace(
     var_t *var, var_parse_t *ctx,
-    tokenbuf_t *data, 
+    tokenbuf_t *data,
     tokenbuf_t *search,
-    tokenbuf_t *replace, 
+    tokenbuf_t *replace,
     tokenbuf_t *flags)
 {
     tokenbuf_t tmp;
@@ -997,7 +998,7 @@ op_search_and_replace(
     if (no_regex) {
         /* plain text pattern based operation */
         tokenbuf_init(&tmp);
-        for (p = data->begin; p != data->end;) {
+        for (p = data->begin; p != data->end; ) {
             if (case_insensitive)
                 rc = strncasecmp(p, search->begin, search->end - search->begin);
             else
@@ -1042,7 +1043,7 @@ op_search_and_replace(
         }
 
         /* compile the pattern. */
-        rc = regcomp(&preg, tmp.begin, 
+        rc = regcomp(&preg, tmp.begin,
                      (  REG_EXTENDED
                       | (multiline ? REG_NEWLINE : 0)
                       | (case_insensitive ? REG_ICASE : 0)));
@@ -1065,8 +1066,8 @@ op_search_and_replace(
                 tokenbuf_append(&tmp, p, mydata.end - p);
                 break;
             }
-            else if (   multiline 
-                     && (p + pmatch[0].rm_so) == mydata.end 
+            else if (   multiline
+                     && (p + pmatch[0].rm_so) == mydata.end
                      && (pmatch[0].rm_eo - pmatch[0].rm_so) == 0) {
                 /* special case: found empty pattern (usually /^/ or /$/ only)
                    in multi-line at end of data (after the last newline) */
@@ -1090,7 +1091,7 @@ op_search_and_replace(
                     return rc;
                 }
                 /* append replace string */
-                if (!tokenbuf_append(&tmp, myreplace.begin, myreplace.end - myreplace.begin)) {
+                if (!tokenbuf_merge(&tmp, &myreplace)) {
                     regfree(&preg);
                     tokenbuf_free(&tmp);
                     tokenbuf_free(&mydata);
@@ -1115,7 +1116,7 @@ op_search_and_replace(
                     }
                     p++;
                 }
-                /* append prolog string and stop processing if we 
+                /* append prolog string and stop processing if we
                    do not perform the search & replace globally */
                 if (!global) {
                     if (!tokenbuf_append(&tmp, p, mydata.end - p)) {
@@ -1138,12 +1139,12 @@ op_search_and_replace(
 }
 
 /* operation: offset substring */
-static int 
+static int
 op_offset(
     var_t *var, var_parse_t *ctx,
-    tokenbuf_t *data, 
+    tokenbuf_t *data,
     int num1,
-    int num2, 
+    int num2,
     int isrange)
 {
     tokenbuf_t res;
@@ -1180,11 +1181,11 @@ op_offset(
 }
 
 /* operation: padding */
-static int 
+static int
 op_padding(
     var_t *var, var_parse_t *ctx,
-    tokenbuf_t *data, 
-    int width, 
+    tokenbuf_t *data,
+    int width,
     tokenbuf_t *fill,
     char position)
 {
@@ -1214,7 +1215,7 @@ op_padding(
         if (i > 0) {
             i = i / (fill->end - fill->begin);
             while (i > 0) {
-                if (!tokenbuf_append(&result, fill->begin, fill->end - fill->begin)) {
+                if (!tokenbuf_merge(&result, fill)) {
                     tokenbuf_free(&result);
                     return VAR_ERR_OUT_OF_MEMORY;
                 }
@@ -1225,7 +1226,7 @@ op_padding(
                 tokenbuf_free(&result);
                 return VAR_ERR_OUT_OF_MEMORY;
             }
-            if (!tokenbuf_append(&result, data->begin, data->end - data->begin)) {
+            if (!tokenbuf_merge(&result, data)) {
                 tokenbuf_free(&result);
                 return VAR_ERR_OUT_OF_MEMORY;
             }
@@ -1240,7 +1241,7 @@ op_padding(
             /* create the prefix */
             i = i / (fill->end - fill->begin);
             while (i > 0) {
-                if (!tokenbuf_append(&result, fill->begin, fill->end - fill->begin)) {
+                if (!tokenbuf_merge(&result, fill)) {
                     tokenbuf_free(&result);
                     return VAR_ERR_OUT_OF_MEMORY;
                 }
@@ -1253,7 +1254,7 @@ op_padding(
                 return VAR_ERR_OUT_OF_MEMORY;
             }
             /* append the actual data string */
-            if (!tokenbuf_append(&result, data->begin, data->end - data->begin)) {
+            if (!tokenbuf_merge(&result, data)) {
                 tokenbuf_free(&result);
                 return VAR_ERR_OUT_OF_MEMORY;
             }
@@ -1261,7 +1262,7 @@ op_padding(
             i = width - (result.end - result.begin);
             i = i / (fill->end - fill->begin);
             while (i > 0) {
-                if (!tokenbuf_append(&result, fill->begin, fill->end - fill->begin)) {
+                if (!tokenbuf_merge(&result, fill)) {
                     tokenbuf_free(&result);
                     return VAR_ERR_OUT_OF_MEMORY;
                 }
@@ -1281,7 +1282,7 @@ op_padding(
 }
 
 /* parse an integer number ("123") */
-static int 
+static int
 parse_integer(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -1303,7 +1304,7 @@ parse_integer(
 }
 
 /* parse an operation (":x...") */
-static int 
+static int
 parse_operation(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -1585,7 +1586,7 @@ parse_operation(
             const char *val_ptr;
             size_t val_len;
             const char *out_ptr;
-            size_t out_len; 
+            size_t out_len;
             size_t out_size;
             tokenbuf_t args;
 
@@ -1625,11 +1626,16 @@ parse_operation(
                                               arg_ptr, arg_len,
                                               val_ptr, val_len,
                                               &out_ptr, &out_len, &out_size);
-                if (rc < 0)
+                if (rc < 0) {
+                    if (arg_ptr != NULL)
+                        free((void *)arg_ptr);
                     goto error_return;
+                }
                 tokenbuf_free(data);
                 tokenbuf_set(data, out_ptr, out_ptr+out_len, out_size);
             }
+            if (arg_ptr != NULL)
+                free((void *)arg_ptr);
             break;
         }
         default:
@@ -1658,10 +1664,10 @@ parse_operation(
 }
 
 /* parse numerical expression operand */
-static int 
+static int
 parse_numexp_operand(
     var_t *var, var_parse_t *ctx,
-    const char *begin, const char *end, 
+    const char *begin, const char *end,
     int *result, int *failed)
 {
     const char *p;
@@ -1721,7 +1727,7 @@ parse_numexp_operand(
         }
     }
     /* parse relative index mark ("#") */
-    else if (   var->syntax.index_mark != NUL 
+    else if (   var->syntax.index_mark != NUL
              && *p == var->syntax.index_mark) {
         p++;
         *result = ctx->index_this;
@@ -1762,14 +1768,14 @@ parse_numexp_operand(
 }
 
 /* parse numerical expression ("x+y") */
-static int 
+static int
 parse_numexp(
     var_t *var, var_parse_t *ctx,
-    const char *begin, const char *end, 
+    const char *begin, const char *end,
     int *result, int *failed)
 {
     const char *p;
-    char operator;
+    char op;
     int right;
     int rc;
 
@@ -1787,27 +1793,27 @@ parse_numexp(
     /* parse numerical operator */
     while (p != end) {
         if (*p == '+' || *p == '-') {
-            operator = *p++;
+            op = *p++;
             /* recursively parse right operand (light binding) */
             rc = parse_numexp(var, ctx, p, end, &right, failed);
             if (rc < 0)
                 return rc;
             p += rc;
-            if (operator == '+')
+            if (op == '+')
                 *result = (*result + right);
             else
                 *result = (*result - right);
         }
         else if (*p == '*' || *p == '/' || *p == '%') {
-            operator = *p++;
+            op = *p++;
             /* recursively parse right operand (string binding) */
             rc = parse_numexp_operand(var, ctx, p, end, &right, failed);
             if (rc < 0)
                 return rc;
             p += rc;
-            if (operator == '*')
+            if (op == '*')
                 *result = (*result * right);
-            else if (operator == '/') {
+            else if (op == '/') {
                 if (right == 0) {
                     if (*failed)
                         *result = 0;
@@ -1817,7 +1823,7 @@ parse_numexp(
                 else
                     *result = (*result / right);
             }
-            else if (operator == '%') {
+            else if (op == '%') {
                 if (right == 0) {
                     if (*failed)
                         *result = 0;
@@ -1837,7 +1843,7 @@ parse_numexp(
 }
 
 /* parse variable name ("abc") */
-static int 
+static int
 parse_name(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
@@ -1851,7 +1857,7 @@ parse_name(
 }
 
 /* lookup a variable value through the callback function */
-static int 
+static int
 lookup_value(
     var_t *var, var_parse_t *ctx,
     const char  *var_ptr, size_t  var_len, int     var_idx,
@@ -1862,13 +1868,13 @@ lookup_value(
 
     /* pass through to original callback */
     rc = (*var->cb_value_fct)(var, var->cb_value_ctx,
-                              var_ptr, var_len, var_idx, 
+                              var_ptr, var_len, var_idx,
                               val_ptr, val_len, val_size);
 
     /* convert undefined variable into empty variable if relative
        lookups are counted. This is the case inside an active loop
        construct if no limits are given. There the parse_input()
-       has to proceed until all variables have undefined values. 
+       has to proceed until all variables have undefined values.
        This trick here allows it to determine this case. */
     if (ctx->rel_lookup_flag && rc == VAR_ERR_UNDEFINED_VARIABLE) {
         ctx->rel_lookup_cnt--;
@@ -1883,7 +1889,7 @@ lookup_value(
 }
 
 /* parse complex variable construct ("${name...}") */
-static int 
+static int
 parse_variable_complex(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -1936,11 +1942,12 @@ parse_variable_complex(
                 rc = VAR_ERR_OUT_OF_MEMORY;
                 goto error_return;
             }
+            tokenbuf_free(&tmp);
             p += rc;
         }
     } while (rc > 0);
 
-    /* we must have the complete expanded variable name now, 
+    /* we must have the complete expanded variable name now,
        so make sure we really do. */
     if (name.begin == name.end) {
         if (ctx->force_expand) {
@@ -2056,7 +2063,7 @@ parse_variable_complex(
 }
 
 /* parse variable construct ("$name" or "${name...}") */
-static int 
+static int
 parse_variable(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -2078,7 +2085,7 @@ parse_variable(
     if (p == end)
         return VAR_ERR_INCOMPLETE_VARIABLE_SPEC;
 
-    /* parse a simple variable name. 
+    /* parse a simple variable name.
        (if this fails, we're try to parse a complex variable construct) */
     rc = parse_name(var, ctx, p, end);
     if (rc < 0)
@@ -2103,7 +2110,7 @@ parse_variable(
 }
 
 /* parse loop construct limits ("[...]{b,s,e}") */
-static var_rc_t 
+static int
 parse_looplimits(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -2200,14 +2207,14 @@ parse_looplimits(
 }
 
 /* parse plain text */
-static int 
+static int
 parse_text(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end)
 {
     const char *p;
 
-    /* parse until delim_init (variable construct) 
+    /* parse until delim_init (variable construct)
        or index_open (loop construct) is found */
     for (p = begin; p != end; p++) {
         if (*p == var->syntax.escape) {
@@ -2218,7 +2225,7 @@ parse_text(
         else if (*p == var->syntax.delim_init)
             break;
         else if (   var->syntax.index_open != NUL
-                 && (   *p == var->syntax.index_open 
+                 && (   *p == var->syntax.index_open
                      || *p == var->syntax.index_close))
             break;
     }
@@ -2226,7 +2233,7 @@ parse_text(
 }
 
 /* expand input in general */
-static var_rc_t 
+static int
 parse_input(
     var_t *var, var_parse_t *ctx,
     const char *begin, const char *end,
@@ -2247,8 +2254,8 @@ parse_input(
 
     do {
         /* try to parse a loop construct */
-        if (   p != end 
-            && var->syntax.index_open != NUL 
+        if (   p != end
+            && var->syntax.index_open != NUL
             && *p == var->syntax.index_open) {
             p++;
 
@@ -2268,24 +2275,24 @@ parse_input(
                or there is a stop limit known and it is still not reached */
             re_loop:
             for (i = start;
-                 (   (   open_stop 
-                      && (   loop_limit_length < 0 
-                          || rel_lookup_cnt > ctx->rel_lookup_cnt)) 
-                  || (   !open_stop 
+                 (   (   open_stop
+                      && (   loop_limit_length < 0
+                          || rel_lookup_cnt > ctx->rel_lookup_cnt))
+                  || (   !open_stop
                       && i <= stop)                                );
                  i += step) {
 
                 /* remember current output end for restoring */
                 output_backup = (output->end - output->begin);
 
-                /* open temporary context for recursion */ 
+                /* open temporary context for recursion */
                 ctx = var_parse_push(ctx, &myctx);
                 ctx->force_expand    = 1;
                 ctx->rel_lookup_flag = 1;
                 ctx->index_this      = i;
 
                 /* recursive parse input through ourself */
-                rc = parse_input(var, ctx, p, end, 
+                rc = parse_input(var, ctx, p, end,
                                  output, recursion_level+1);
 
                 /* retrieve info and close temporary context */
@@ -2304,7 +2311,7 @@ parse_input(
 
                 /* try to parse loop construct limit specification */
                 if (loop_limit_length < 0) {
-                    rc2 = parse_looplimits(var, ctx, p+rc+1, end, 
+                    rc2 = parse_looplimits(var, ctx, p+rc+1, end,
                                            &start, &step, &stop, &open_stop);
                     if (rc2 < 0)
                         goto error_return;
@@ -2357,7 +2364,7 @@ parse_input(
             tokenbuf_free(&result);
             p += rc;
             continue;
-        }    
+        }
         tokenbuf_free(&result);
         if (rc < 0)
             goto error_return;
@@ -2387,14 +2394,14 @@ parse_input(
     return rc;
 }
 
-/* 
+/*
 **
 **  ==== APPLICATION PROGRAMMING INTERFACE (API) ====
 **
 */
 
 /* create variable expansion context */
-var_rc_t 
+var_rc_t
 var_create(
     var_t **pvar)
 {
@@ -2411,7 +2418,7 @@ var_create(
 }
 
 /* destroy variable expansion context */
-var_rc_t 
+var_rc_t
 var_destroy(
     var_t *var)
 {
@@ -2422,10 +2429,10 @@ var_destroy(
 }
 
 /* configure variable expansion context */
-var_rc_t 
+var_rc_t
 var_config(
-    var_t *var, 
-    var_config_t mode, 
+    var_t *var,
+    var_config_t mode,
     ...)
 {
     va_list ap;
@@ -2450,7 +2457,7 @@ var_config(
             var->syntax.name_chars  = NULL; /* unused internally */
             if ((rc = expand_character_class(s->name_chars, var->syntax_nameclass)) != VAR_OK)
                 return VAR_RC(rc);
-            if (   var->syntax_nameclass[(int)var->syntax.delim_init] 
+            if (   var->syntax_nameclass[(int)var->syntax.delim_init]
                 || var->syntax_nameclass[(int)var->syntax.delim_open]
                 || var->syntax_nameclass[(int)var->syntax.delim_close]
                 || var->syntax_nameclass[(int)var->syntax.escape])
@@ -2483,11 +2490,11 @@ var_config(
 }
 
 /* perform unescape operation on a buffer */
-var_rc_t 
+var_rc_t
 var_unescape(
-    var_t *var, 
-    const char *src, size_t srclen, 
-    char *dst, size_t dstlen, 
+    var_t *var,
+    const char *src, size_t srclen,
+    char *dst, size_t dstlen,
     int all)
 {
     const char *end;
@@ -2523,8 +2530,8 @@ var_unescape(
                     break;
                 case '0': case '1': case '2': case '3': case '4':
                 case '5': case '6': case '7': case '8': case '9':
-                    if (   end - src >= 3 
-                        && isdigit((int)src[1]) 
+                    if (   end - src >= 3
+                        && isdigit((int)src[1])
                         && isdigit((int)src[2])) {
                         if ((rc = expand_octal(&src, &dst, end)) != 0)
                             return VAR_RC(rc);
@@ -2545,11 +2552,11 @@ var_unescape(
 }
 
 /* perform expand operation on a buffer */
-var_rc_t 
+var_rc_t
 var_expand(
-    var_t *var, 
-    const char *src_ptr, size_t src_len, 
-    char **dst_ptr, size_t *dst_len, 
+    var_t *var,
+    const char *src_ptr, size_t src_len,
+    char **dst_ptr, size_t *dst_len,
     int force_expand)
 {
     var_parse_t ctx;
@@ -2569,11 +2576,11 @@ var_expand(
 
     /* start the parsing */
     tokenbuf_init(&output);
-    rc = parse_input(var, &ctx, src_ptr, src_ptr+src_len, &output, 0);
+    rc = (var_rc_t)parse_input(var, &ctx, src_ptr, src_ptr+src_len, &output, 0);
 
     /* post-processing */
     if (rc >= 0) {
-        /* always NUL-terminate output for convinience reasons 
+        /* always NUL-terminate output for convinience reasons
            but do not count the NUL-terminator in the length */
         if (!tokenbuf_append(&output, "\0", 1)) {
             tokenbuf_free(&output);
@@ -2598,10 +2605,10 @@ var_expand(
 }
 
 /* format and expand a string */
-var_rc_t 
+var_rc_t
 var_formatv(
-    var_t *var, 
-    char **dst_ptr, int force_expand, 
+    var_t *var,
+    char **dst_ptr, int force_expand,
     const char *fmt, va_list ap)
 {
     var_rc_t rc;
@@ -2642,10 +2649,10 @@ var_formatv(
 }
 
 /* format and expand a string */
-var_rc_t 
+var_rc_t
 var_format(
-    var_t *var, 
-    char **dst_ptr, int force_expand, 
+    var_t *var,
+    char **dst_ptr, int force_expand,
     const char *fmt, ...)
 {
     var_rc_t rc;
@@ -2717,8 +2724,8 @@ var_rc_t var_strerror(var_t *var, var_rc_t rc, char **str)
 {
     if (str == NULL)
         return VAR_RC(VAR_ERR_INVALID_ARGUMENT);
-    rc = 0 - rc;
-    if (rc < 0 || rc >= sizeof(var_errors) / sizeof(char *))
+    rc = (var_rc_t)(0 - rc);
+    if ((int)rc < 0 || (int)rc >= (int)(sizeof(var_errors) / sizeof(char *)))
         *str = "unknown error";
     else
         *str = (char *)var_errors[rc];
